#!/bin/bash
VERSION="1.0.0"

VERSION_SIA="latest"
AUTO_YES=0
PASSWORD=""
WEB_PORT=0
INSTALL_DOCKER=-1
INSTALL_EDGE=-1
INSTALL_USERSCRIPTS=-1

echo -e "\n"
echo "# ----------------------------------------------------------------------- #"
echo "#                    SIA Connect installer ($VERSION)                        #"
echo "# ----------------------------------------------------------------------- #"

echo -e "\n"

echo "# ----------------------------------------------------------------------- #"
echo "#                            Changelog                                    #"
echo "#                                                                         #"
echo "#                          Version 1.0.0                                  #"
echo "# - Initial edition of installation script                                #"
echo "#                                                                         #"
echo "# ----------------------------------------------------------------------- #"


echo -e "\n\n"

ARCH=$(uname -mi)

# Parsing arguments
while getopts ":yp:v:w:d:e:" opt; do
  case $opt in
    y) AUTO_YES=1;
	echo "Installation with auto-yes";
    ;;
	p) PASSWORD=$OPTARG;
	echo "Password set to $PASSWORD";
    ;;
	v) VERSION_SIA=$OPTARG;
	echo "Using version set to $VERSION_SIA";
    ;;
	w) WEB_PORT=$OPTARG;
	echo "Web port for GUI set to $WEB_PORT";
    ;;
	d) INSTALL_DOCKER=$OPTARG;
	;;
	e) INSTALL_EDGE=$OPTARG;
	;;
	u) INSTALL_USERSCRIPTS=$OPTARG;
	;;
    \?) echo "Invalid option -$OPTARG" >&2
    exit 1
    ;;
  esac
done

#Functions
clean_up () {
	rm -Rf files
	rm -Rf tmp
}

#This script needs root access. Checking for root
if [ "$EUID" -ne 0 ]
  then echo "Please run SIA Connect installation script as root"
  exit
fi

# Docker check
IS_DOCKER=0
if [ -f /.dockerenv ] && [ $AUTO_YES = 0 ]; then
	IS_DOCKER=1
	read -p "You are running SIA Connect installation inside a Docker. You can also get SIA Connect as a native Docker image. Do you want to continue to install? (y/n) " yn
	case $yn in 
		[yY] ) echo Continuing installation;
			;;
		[nN] ) echo You can download Docker from https://link.sia-connect.com/software-docker - Exiting;
			exit;
			;;
		* ) echo invalid response: $yn
			;;
	esac
fi

echo "Welcome to the installation script for SIA Connect edge computing"

# Password check
if [ -z $PASSWORD ]; then
	echo "Enter password to use for the SIA Connect installation: "
	read -s P 
	echo

	# Repeat
	echo "Please repeat password: "
	read -s PD 
	echo

	# Validate
	if [ $PD != $P ]; then
		echo "Password does not match"
		exit
	fi

	PASSWORD=$P
fi

unset autostart_yn
if [ $AUTO_YES = 0 ] && [ ! IS_DOCKER ]; then #Not working on containers
	read -p "Do you want to add SIA Connect to autostart on boot? (y/n) " autostart_yn
else
	autostart_yn="y"
fi


unset web_port_yn
if [ $AUTO_YES = 0 ] && [ $WEB_PORT = 0 ]; then
	WEB_PORT=80
	read -p "Do you want to disable Apache2 default configuration and use port 80 for SIA Connect web GUI? Selecting \"no\" will provide you the option to select a custom port (y/n) " web_port_yn

	case $web_port_yn in 
		[yY] ) echo "Yes, enable SIA Connect to run its GUI on web port $WEB_PORT";
			;;
		[nN] ) echo "No, select another web port for SIA Connect GUI";
			read -p 'Port: ' input;
			if [[ $input ]] && [ $input -eq $input 2>/dev/null ] 
			then
				WEB_PORT=$input
			else
				echo "$input is not a valid port number. Please input a valid numeric range between 0 and 65536"
			fi
			;;
		* ) echo invalid response
			;;
	esac	

elif [ $WEB_PORT != 0 ]; then
	web_port_yn="y"
else
	web_port_yn="y"
fi



# Check if Docker is installed on the edge. Only if we are not already inside a docker and INSTALL_DOCKER was not set to 0 in the parameter
unset install_docker_yn
if [ ! -x "$(command -v docker)" ] && [ $INSTALL_DOCKER != 0 ]; then

	if [ $AUTO_YES = 0 ] && [ $INSTALL_DOCKER = -1 ]; then #Ask to install if parameter or auto yes is not already set by the user
		read -p  "Docker not installed on host. Do you want to install it? (y/n) " install_docker_yn
	else
		install_docker_yn="y"
	fi
    
    case $install_docker_yn in 
		[yY] ) echo Install Docker;
			INSTALL_DOCKER=1
			;;
		[nN] ) echo "Continuing without installing Docker";
			INSTALL_DOCKER=0
			;;
		* ) echo invalid response: $install_docker_yn;;
	esac
fi

unset edge_components_yn
if [[ $AUTO_YES = 0 && $INSTALL_EDGE = -1 && (-x "$(command -v docker)" || $INSTALL_DOCKER = 1) ]]; then
	read -p "Do you want to install edge component to the host? This include Portainer CE for container management (y/n) " edge_components_yn
elif [ $INSTALL_EDGE = 1 ] || [ $AUTO_YES = 1 ]; then
	edge_components_yn="y"
elif [ $INSTALL_EDGE = 0 ]; then
	edge_components_yn="n"
fi

# unset user_scripts_yn
# if [ $AUTO_YES = 0 ] && [ $INSTALL_USERSCRIPTS = -1 ]; then
# 	read -p "Do you want to install user scripts? This will install all scripts inside the folder user_scripts (y/n) " user_scripts_yn
# elif [ $INSTALL_USERSCRIPTS = 1 ] || [ $AUTO_YES = 1 ]; then
# 	user_scripts_yn="y"
# elif [ $INSTALL_USERSCRIPTS = 0 ]; then
# 	user_scripts_yn="n"
# fi
user_scripts_yn="y"
INSTALL_USERSCRIPTS=1


# unset clean_yn
# if [ $AUTO_YES = 0 ]; then
# 	read -p "Do you want to delete all downloaded files and clean up after installation? (y/n) " clean_yn
# else
# 	clean_yn="y"
# fi
clean_yn="y"

#Clear eventual old tmp folder
rm -Rf tmp/
mkdir -p files/
mkdir -p tmp/

# ----------------------------------------------------------------------- #
#                   Installing and setting up Mariadb                     #
# ----------------------------------------------------------------------- #

# Install mariadb and setup tabelstructure
echo "Installing SIA Connect database"
DEBIAN_FRONTEND=noninteractive apt update && apt -y install mariadb-server curl
# Start mariadb service
service mariadb start

#Setting the root password and use mysql_native_password instead of the auth_socket which requires the unix user as user
#Only do this if the user has no password set. If password is set, the installation has already been done previously
#TODO not working if [ mysql -u root -e 'quit' &> /dev/null ]; then
  mysql -e "UPDATE mysql.user SET Password = PASSWORD('$PASSWORD'), plugin='mysql_native_password' WHERE User = 'root'"
  
  #Remove anonymous users
  mysql -e "DROP USER ''@'localhost';"
  mysql -e "DROP USER ''@'$(hostname)';"
  
  #Drop the test database
  mysql -e "DROP DATABASE test;"
  
  #Flush priveleges with changes
  mysql -e "FLUSH PRIVILEGES;"
#fi


# Import tabels and static settings to database.
if [ ! -f files/sia-connect-sql-conf.tar.gz ]; then
	if ! curl -sL --fail https://siaartifacts.blob.core.windows.net/core/siasqlconfiguration.tar.gz > files/sia-connect-sql-conf.tar.gz; then
		echo "SQL configuration not found from online repository. Please validate you are online. Exiting!"
		clean_up
		exit;
	fi
fi

tar -xf files/sia-connect-sql-conf.tar.gz -C tmp/

mysql --user=root --password=$PASSWORD -e "CREATE DATABASE IF NOT EXISTS box;"
mysql --host=localhost --user=root --password=$PASSWORD box < tmp/sql/CreateTables.sql
mysql --host=localhost --user=root --password=$PASSWORD box < tmp/sql/PopulateStaticSettings.sql

# # ----------------------------------------------------------------------- #
# #                   Installing and configuring apache2                    #
# # ----------------------------------------------------------------------- #
echo "Installing apache2"
DEBIAN_FRONTEND=noninteractive apt -y install apache2

# # ----------------------------------------------------------------------- #
# #                   Installing and setting up php and phpMyAdmin          #
# # ----------------------------------------------------------------------- #

DEBIAN_FRONTEND=noninteractive apt update 
DEBIAN_FRONTEND=noninteractive apt -y install php-mysql php-xml php-zip libapache2-mod-php php-mbstring

# # # install phpMyAdmin
DEBIAN_FRONTEND=noninteractive  apt -y install phpmyadmin

mv /etc/phpmyadmin/apache.conf /etc/apache2/sites-enabled/phpmyadmin.conf
sed -i "s/Alias \/phpmyadmin/Alias \/service\/database/g" /etc/apache2/sites-enabled/phpmyadmin.conf #change alias to /service/database

#echo "Apache2 enable needed modules"
a2enmod headers
a2enmod php
a2enmod ssl
a2enmod rewrite

# # ----------------------------------------------------------------------- #
# #          Installing and setting up SIA Core & System libs               #
# # ----------------------------------------------------------------------- #

apt update && \
	DEBIAN_FRONTEND=noninteractive apt install -y \
		udev \
		libuuid1 \
		python3 python3-netifaces \
		unzip \
		net-tools \
		wireless-tools \
		systemd \
		inetutils-ping \
		dmidecode \
		ifupdown


if [ $INSTALL_DOCKER = 1 ]; then
	#Installing Docker and giving access to www-data user to execute Docker commands. Debian package already executes this usermod command, maybe not necessary here?
	DEBIAN_FRONTEND=noninteractive curl -fsSL https://get.docker.com -o files/get-docker.sh \
		 && sh files/get-docker.sh

	#Give access for the www-data (Apache) to view containers from Docker
	if [ $(getent group docker) ]; then
		usermod -aG docker www-data 2> /dev/null
	fi
fi

#Ensure python command is available
DEBIAN_FRONTEND=noninteractive update-alternatives --install /usr/bin/python python /usr/bin/python3 1


# SIA Connect engine is packed for 32bit, so we have to enable this
if [[ ${ARCH,,} = x86_64* ]]; then	
 	DEBIAN_FRONTEND=noninteractive dpkg --add-architecture i386
 	DEBIAN_FRONTEND=noninteractive apt update
fi

if [[ ${ARCH,,} = arm64* || ${ARCH,,} = aarch64* ]]; then	
 	DEBIAN_FRONTEND=noninteractive dpkg --add-architecture armhf
 	DEBIAN_FRONTEND=noninteractive apt update
	DEBIAN_FRONTEND=noninteractive apt install -y libc6:armhf #libncurses5:armhf libstdc++6:armhf
fi

#Download the installer and unpack it dependent on the architecture
if [ ! -f files/sia-connect.siaup ]; then
	if [[ ${ARCH,,} = x86* || ${ARCH,,} = i*86* ]]; then

		#PostgreSQL and MariaDB libraries are not part of the host library package. Install those
		DEBIAN_FRONTEND=noninteractive apt install -y \
			libmariadb-dev \
			libpq-dev \
			unixodbc-dev \
			tdsodbc

		if ! curl -sL --fail https://siadownloads.blob.core.windows.net/system/sia-update-$VERSION_SIA-x86.siaup > files/sia-connect.siaup; then
			echo "Version: $VERSION_SIA, does not exists (x86). Exiting!"
			clean_up
			exit
		fi
	# Not yet support for 64 bit. Therefore use the hardware version which contains all the libraries for 32 bit so it can run on a 64bit.
	elif [[ ${ARCH,,} = arm64* || ${ARCH,,} = aarch64* ]]; then
		if ! curl -sL --fail https://siadownloads.blob.core.windows.net/system/sia-update-$VERSION_SIA-arm-sia-v1.siaup > files/sia-connect.siaup; then
			echo "Version: $VERSION_SIA, does not exists (ARM). Exiting!"
			clean_up
			exit;
		fi
	# 32 bit architecture
	elif [[ ${ARCH,,} = arm* || ${ARCH,,} = aarch* ]]; then
		if ! curl -sL --fail https://siadownloads.blob.core.windows.net/system/sia-update-$VERSION_SIA-arm.siaup > files/sia-connect.siaup; then
			echo "Version: $VERSION_SIA, does not exists (ARM). Exiting!"
			clean_up
			exit;
		fi
	else
		echo "Unsupported architecture: $ARCH"
		exit
	fi
fi

unzip files/sia-connect.siaup -d tmp/

#Run the installation
chmod +x tmp/simple_install.sh
./tmp/simple_install.sh ./tmp $PASSWORD


#Create symbolic link for web & api
ln -s /opt/SIA/api/siaapi /opt/SIA/api/main 2>/dev/null
ln -s /opt/SIA/web/siaweb /opt/SIA/web/main 2>/dev/null

#Check if old SIA Connect installation exists. If not, then make a set of configuration files
if [ ! -f files/sia-connect-conf.tar.gz ]; then
	if [ ! -f /opt/SIA/engine/SIA.conf ]; then
		curl -s -L https://siaartifacts.blob.core.windows.net/core/siaconfigurationfiles.tar.gz > files/sia-connect-conf.tar.gz
		curl -s -L https://siaartifacts.blob.core.windows.net/core/siaapiconfigurationfiles.tar.gz > files/sia-connect-api-conf.tar.gz
		tar -xf files/sia-connect-conf.tar.gz -C /opt/SIA/
		tar -xf files/sia-connect-api-conf.tar.gz -C /opt/SIA/
	fi
fi

# Set the password in all the configuration files
python /opt/SIA/scripts/set_sia_passwords.py $PASSWORD $PASSWORD

case $autostart_yn in 
	[yY] ) echo Adding SIA Connect to autostart on boot;
		update-rc.d mariadb defaults 2>/dev/null;
		update-rc.d mysql defaults;
		update-rc.d apache2 defaults;
		;;
	[nN] ) echo Not adding SIA Connect to autostart on boot. Continuining;
		;;
	* ) echo invalid response: $autostart_yn
		;;
esac


#Disable default apache2 configuration so we are using main
if [ $WEB_PORT != 80 ]; then
	web_port_yn="y"
	sed -i "s/:80/:$WEB_PORT/g" /etc/apache2/sites-enabled/siaweb.conf
else
	a2dissite 000-default;
fi


# ----------------------------------------------------------------------- #
#                    Edge components (Portainer, etc.)                    #
# ----------------------------------------------------------------------- #
case $edge_components_yn in 
	[yY] ) echo "Installing edge components";
		INSTALL_EDGE=1
		;;
	[nN] ) echo "Continue without installing edge components";
		;;
	* ) echo invalid response;;
esac

if [ $INSTALL_EDGE = 1 ]; then
	if [ ! -f files/sia-connect-edge-components-installer-linux.sh ]; then
		if ! curl -sL --fail https://siadownloads.blob.core.windows.net/software/sia-connect-edge-components-installer-linux.sh > files/sia-connect-edge-components-installer-linux.sh; then
			echo "Edge component installer script not found from online repository. Please validate you are online. Exiting!"
			clean_up
			exit;
		fi
	fi

	chmod +x files/sia-connect-edge-components-installer-linux.sh
	./files/sia-connect-edge-components-installer-linux.sh
fi


# ----------------------------------------------------------------------- #
#                   Starting services of SIA Connect                      #
# ----------------------------------------------------------------------- #
echo Starting SIA Connect and dependency services

service apache2 restart
service mariadb restart
service siacore restart
service siawatchdog restart


# ----------------------------------------------------------------------- #
#               		      User scripts                                #
# ----------------------------------------------------------------------- #

#Export variables as it might be the user scripts needs them
export SIA_CONNECT_PASSWORD=$PASSWORD
export SIA_CONNECT_VERSION=$VERSION_SIA
export SIA_CONNECT_WEB_PORT=$WEB_PORT
export SIA_CONNECT_ARCH=$ARCH

case $user_scripts_yn in 
	[yY] ) echo "Installing user scripts";
		INSTALL_USERSCRIPTS=1
		;;
	[nN] ) echo "Continue without installing user scripts";
		;;
	* ) echo invalid response;;
esac

# User scripts located in user_scripts directory
if [ $INSTALL_USERSCRIPTS = 1 ]; then
	for f in user_scripts/*.sh; do
		echo "Executing user script: $f"
		bash "$f" 
	done
fi


# ----------------------------------------------------------------------- #
#                 			    Clean up				                  #
# ----------------------------------------------------------------------- #
case $clean_yn in 
	[yY] ) echo "Yes, remove all files";
		clean_up
		;;
	[nN] ) echo "No, keep them";
		;;
	* ) echo invalid response: $clean_yn
		;;
esac